@@ -28,11 +28,17 @@
 to keep the state of a model object synchronized with a View.
 """
 
-import sys
+import gobject
 
-from kiwi import _warn, ValueUnset
+from kiwi import ValueUnset
 from kiwi.accessors import kgetattr, ksetattr, clear_attr_cache
 from kiwi.interfaces import Mixin, MixinSupportValidation
+from kiwi.log import Logger
+
+class ProxyError(Exception):
+    pass
+
+log = Logger('proxy')
 
 def block_widget(widget):
     """Blocks the signal handler of the 'content-changed' signal on widget"""
@@ -55,12 +61,28 @@ class Proxy:
     attached to the model by looking if it is a KiwiWidget and if it
     has the model-attribute set.
     """
-    _setter_error_handler = None
     
     def __init__(self, view, model=None, widgets=[]):
-        self.view = view
+        """
+        @param view:    view attched to the slave
+        @type  view:    a L{kiwi.ui.views.BaseView} subclass
+        @param model:   model attached to proxy
+        @param widgets: the widget names
+        @type  widgets: list of strings
+        """
+        self._view = view
         self.model = model
-        self._setup_widgets(widgets)
+        self._model_attributes = {}
+
+        for widget_name in widgets:
+            widget = getattr(self._view, widget_name, None)
+            if widget is None:
+                raise AttributeError("The widget %s was not found in the "
+                                     "view %s" % (
+                    widget_name, self._view.__class__.__name__))
+            
+            self._setup_widget(widget_name, widget)
+            
         self._initialize_widgets()
 
     def _initialize_widgets(self):
@@ -68,7 +90,7 @@ class Proxy:
 
         This should be called after _setup_widgets.
         """
-        for attribute, widget in self._attr_map.items():
+        for attribute, widget in self._model_attributes.items():
 
             if self.model is None:
                 # if we have no model, leave value unset so we pick up
@@ -78,114 +100,95 @@ class Proxy:
                 # if we have a model, grab its value to update the widgets
                 self._register_proxy_in_model(attribute)
                 value = kgetattr(self.model, attribute, ValueUnset)
-                if value is ValueUnset:
-                    # get the default value from the widget if it has any
-                    defvalue = widget.get_property('default-value')
-                    if defvalue is not None:
-                        value = defvalue
-
-            tweak_function = getattr(self, "tweak_%s" % attribute, None)
-            if tweak_function:
-                tweak_function(attribute, value)
-            else:
-                self.update(attribute, value, block=True)
+                    
+            self.update(attribute, value, block=True)
 
             # The initial value of the model is set, at this point
             # do a read, it'll trigger a validation for widgets who
             # supports it.
             if not isinstance(widget, MixinSupportValidation):
                 continue
-            
-            data = widget.read()
-            if data is not ValueUnset:
-                widget.validate_data(data, force=True)
 
-    def _setup_widgets(self, widgets):
-        """
-        Connect to the 'content-changed' signal of all the Kiwi widgets
-        in the widgets list parameter.
-        @param widgets: the widget names
-        @type  widgets: list of strings
-        """
-        self._attr_map = {}
-        for widget_name in widgets:
-            widget = getattr(self.view, widget_name, None)
-            if widget is None:
-                raise AttributeError("The widget %s was not "
-                                     "found in the view %s" % (widget_name,
-                                                               self.view))
+            widget.validate(force=True)
             
-            if not isinstance(widget, Mixin):
-                continue
+    def _setup_widget(self, widget_name, widget):
+        if not isinstance(widget, Mixin):
+            raise ProxyError("The widget %s (%r), in view %s is not "
+                             "a kiwi widget and cannot be added to a proxy"
+                             % (widget_name, widget,
+                                self._view.__class__.__name__))
 
-            data_type = widget.get_property('data-type')
-            if data_type is None:
-                raise TypeError("The KiwiWidget %s should have a data type "
-                                "set up" % widget)
-            
-            attribute = widget.get_property('model-attribute')
-            if not attribute:
-                # we don't listen for changes in this widget because
-                # we don't know the model attribute
-                _warn("The widget %s (%s) is a KiwiWidget but does not have "
-                      "a model attribute set so it will not be eassociated "
-                      "with the model" % (widget, widget.name))
-                continue
-             
-            connection_id = widget.connect('content-changed',
-                                           self._on_widget__content_changed,
-                                           attribute)
-            widget.set_data('content-changed-id', connection_id)
+        data_type = widget.get_property('data-type')
+        if data_type is None:
+            raise ProxyError("The kiwi widget %s (%r) in view %s should "
+                             "have a data type set" % (
+                widget_name, widget, self._view.__class__.__name__))
 
-            # save this widget in our map
-            self._attr_map[attribute] = widget
-            
-            # here we define the view that owns the widget
-            widget.owner = self.view
+        attribute = widget.get_property('model-attribute')
+        if not attribute:
+            raise ProxyError(
+                "The widget %s (%s) in view %s is a kiwi widget but does "
+                "not have a model attribute set so it will not be "
+                "associated with the model" % (
+                widget_name, widget, self._view.__class__.__name__))
 
-    def _on_widget__content_changed(self, widget, attribute):
+        # Do a isinstance here instead of in the callback,
+        # as an optimization, it'll never change in runtime anyway
+        connection_id = widget.connect(
+            'content-changed',
+            self._on_widget__content_changed,
+            attribute,
+            isinstance(widget, MixinSupportValidation))
+        widget.set_data('content-changed-id', connection_id)
+
+        model_attributes = self._model_attributes
+        # save this widget in our map
+        if (attribute in model_attributes and
+            # RadioButtons are allowed several times
+            not gobject.type_is_a(widget, 'GtkRadioButton')):
+            old_widget = model_attributes[attribute]
+            raise KeyError("The widget %s (%r) in view %s is already in "
+                           "the proxy, defined by widget %s (%r)" % (
+                widget_name, widget, self._view.__class__.__name__,
+                old_widget.name, old_widget))
+
+        model_attributes[attribute] = widget
+
+        # here we define the view that owns the widget
+        widget.owner = self._view
+
+    def _on_widget__content_changed(self, widget, attribute, validate):
         """This is called as soon as the content of one of the widget
         changes, the widgets tries fairly hard to not emit when it's not
         neccessary"""
-        
+
         # skip updates for model if there is none, right?
         if self.model is None:
             return
 
-        value = widget.read()
+        if validate:
+            value = widget.validate()
+        else:
+            value = widget.read()
+        
+        log('%s.%s = %r' % (self.model.__class__.__name__,
+                            attribute, value)) 
         
         # only update the model if the data is correct
         if value is ValueUnset:
             return
 
-        if isinstance(widget, MixinSupportValidation):
-            value = widget.validate_data(value)
-        
-        # XXX: one day we might want to queue and unique updates?
-        self._block_proxy_in_model(True)
-        try:
-            ksetattr(self.model, attribute, value)
-        except:
-            if self._setter_error_handler:
-                self._setter_error_handler(sys.exc_value, self.model, 
-                                           attribute, value)
-            else:
-                raise
-
-        self._block_proxy_in_model(False)
-
-        # Call global update hook 
-        self.proxy_updated(widget, value)
-
-    def _block_proxy_in_model(self, state):
         model = self.model
-        if not hasattr(model, "block_proxy"):
-            return
-        
-        if state:
+        # XXX: one day we might want to queue and unique updates?
+        if hasattr(model, "block_proxy"):
             model.block_proxy(self)
-        else:
+            ksetattr(model, attribute, value)
             model.unblock_proxy(self)
+        else:
+            ksetattr(model, attribute, value)
+            
+        # Call global update hook 
+        self.proxy_updated(widget, attribute, value)
 
     def _register_proxy_in_model(self, attribute):
         model = self.model
@@ -209,46 +212,35 @@ class Proxy:
             self.model.unregister_proxy(self)
 
     # Public API
-    def proxy_updated(self, widgetproxy, value):
+    def proxy_updated(self, widget, attribute, value):
         """ This is a hook that is called whenever the proxy updates the
         model. Implement it in the inherited class to perform actions that
         should be done each time the user changes something in the interface.
         This hook by default does nothing.
+        @param widget:
+        @param attribute:
+        @param value:
         """
-        pass
-
-    def set_setter_error_handler(self, handler):
-        """ Sets a method that will be called if an update to a model
-        attribute raises an exception. The handler will receive the
-        following arguments, in order:
-
-            - Exception object
-            - model
-            - attribute name
-            - value attempted
-        """
-        self._setter_error_handler = handler
 
     def update(self, attribute, value=ValueUnset, block=False):
         """ Generic frontend function to update the contentss of a widget based
         on its model attribute name using the internal update functions. 
 
-            - attribute: the name of the attribute whose widget we wish to
-              updated.  If accessing a radiobutton, specify its group
-              name. 
-            - value specifies the value to set in the widget. If
-              unspecified, it defaults to the current model's value
-              (through an accessor, if it exists, or getattr). 
-            - block defines if we are to block cascading proxy updates
-              triggered by this update. You should use block if you are
-              calling update on *the same attribute that is currently
-              being updated*.
-
-              This means if you have hooked to a signal of the widget
-              associated to that attribute, and you call update() for
-              the *same attribute*, use block=True. And pray. 8). If
-              block is set to False, the normal update mechanism will
-              occur (the model being updated in the end, hopefully).
+        @param attribute: the name of the attribute whose widget we wish to
+          updated.  If accessing a radiobutton, specify its group
+          name. 
+        @param value: specifies the value to set in the widget. If
+          unspecified, it defaults to the current model's value
+          (through an accessor, if it exists, or getattr). 
+        @param block: defines if we are to block cascading proxy updates
+          triggered by this update. You should use block if you are
+          calling update on *the same attribute that is currently
+          being updated*.
+          This means if you have hooked to a signal of the widget
+          associated to that attribute, and you call update() for
+          the *same attribute*, use block=True. And pray. 8). If
+          block is set to False, the normal update mechanism will
+          occur (the model being updated in the end, hopefully).
         """
 
         if value is ValueUnset:
@@ -263,7 +255,7 @@ class Proxy:
                 return
             value = kgetattr(self.model, attribute, ValueUnset)
 
-        widget = self._attr_map.get(attribute, None)
+        widget = self._model_attributes.get(attribute, None)
 
         if widget is None:
             raise AttributeError("Called update for `%s', which isn't "
@@ -271,7 +263,8 @@ class Proxy:
                                  "attributes are: %s (you may have "
                                  "forgetten to add `:' to the name in "
                                  "the widgets list)" 
-                                 % (attribute, self, self._attr_map.keys()))
+                                 % (attribute, self,
+                                    self._model_attributes.keys()))
 
         
         # The type of value should match the data-type property. The two
@@ -289,56 +282,38 @@ class Proxy:
         
         if block:
             block_widget(widget)
-            self.view.handler_block(widget)
+            self._view.handler_block(widget)
             widget.update(value)
-            self.view.handler_unblock(widget)
+            self._view.handler_unblock(widget)
             unblock_widget(widget)
         else:
             widget.update(value)
         return True
 
-
-    def notify(self, attribute, value=ValueUnset):
-        """  Notifies the proxy that the named attribute has changed. Calls
-        tweak_foo by default if the attribute provided is "foo" and update_foo
-        exists. This should *only* be called by the FrameWork Model, not by an
-        end-user callback.
-
-            - name: the name of the attribute being changed
-            - value: what it was set to
-        """
-        func = getattr(self, "tweak_%s" % attribute, None)
-        if func:
-            # If value is unset, send in model value to tweak_%s
-            if value is ValueUnset:
-                value = kgetattr(self.model, attribute, ValueUnset)
-                if value is ValueUnset:
-                    raise ValueError("model value for %s was unset" % attribute)
-            func(attribute, value)
-        else:
-            self.update(attribute, value, block=True)
-
     def new_model(self, new_model, relax_type=False):
         """ Reuses the same proxy with another instance as model. Allows a
         proxy interface to change model without the need to destroy and
         recreate the UI (which would cause flashing, at least)
+        @param new_model:
+        @param relax_type:
         """
-        # unregister previous proxy
-        self._unregister_proxy_in_model()
         
+        if self.model is not None:
+            if (not relax_type and
+                type(new_model) != type(self.model) and
+                not isinstance(new_model, self.model.__class__)):
+                raise TypeError("New model has wrong type %s, expected %s"
+                                % (type(new_model), type(self.model)))
+
         # the following isn't strictly necessary, but it currently works
         # around a bug with reused ids in the attribute cache and also
         # makes a lot of sense for most applications (don't want a huge
         # eternal cache pointing to models that you're not using anyway)
         clear_attr_cache()
 
-        if self.model is not None:
-            assert self.model.__class__
-            if not relax_type and type(new_model) != type(self.model) and \
-                not isinstance(new_model, self.model.__class__):
-                raise TypeError("New model has wrong type %s, expected %s"
-                                % (type(new_model), type(self.model)))
-
+        # unregister previous proxy
+        self._unregister_proxy_in_model()
+        
         self.model = new_model
 
         self._initialize_widgets()
