@@ -38,13 +38,14 @@ if (gobject.pygtk_version >= (2, 7, 0) and
 else:
     metabase = type
 
-__all__ = ['ClassInittableObject']
-
-class _ClassInittableMetaType(metabase):
-    def __init__(cls, name, bases, namespace):
-       super(_ClassInittableMetaType, cls).__init__(name, bases, namespace)
-       cls.__class_init__(namespace)
+__all__ = ['ClassInittableMetaType', 'ClassInittableObject']
 
+class ClassInittableMetaType(metabase):
+    def __new__(meta, name, bases, namespace):
+        cls = metabase.__new__(meta, name, bases, namespace)
+        cls.__class_init__(namespace)
+        return cls
+    
 class ClassInittableObject(object):
     """
     I am an object which will call a classmethod called
@@ -56,7 +57,7 @@ class ClassInittableObject(object):
     It's called after the class is created, but before it is put
     in the namespace of the module where it is defined.
     """
-    __metaclass__ = _ClassInittableMetaType
+    __metaclass__ = ClassInittableMetaType
 
     def __class_init__(cls, namespace):
         """
@@ -68,11 +69,14 @@ class ClassInittableObject(object):
         """
     __class_init__ = classmethod(__class_init__)
 
-
 # copied from twisted/python/reflect.py
 def namedAny(name):
     """Get a fully named package, module, module-global object, or attribute.
+    
+    @param name:
+    @returns: object, module or attribute
     """
+    
     names = name.split('.')
     topLevelPackage = None
     moduleNames = names[:]
@@ -102,6 +106,12 @@ def namedAny(name):
         
     return obj
 
+def qual(klass):
+    """
+    @returns: fully qualified module and class name
+    """
+    return klass.__module__ + '.' + klass.__name__
+
 def clamp(x, low, high):
     """
     Ensures that x is between the limits set by low and high.
@@ -113,6 +123,7 @@ def clamp(x, low, high):
     @param    x: the value to clamp.
     @param  low: the minimum value allowed.
     @param high: the maximum value allowed.
+    @returns: the clamped value
     """
     
     return min(max(x, low), high)
@@ -121,6 +132,8 @@ def slicerange(slice, limit):
     """Takes a slice object and returns a range iterator
 
     @param slice: slice object
-    @param limit: maximum value allowed"""
+    @param limit: maximum value allowed
+    @returns: iterator
+    """
     
     return xrange(*slice.indices(limit))
