@@ -30,10 +30,12 @@ Defines the View classes that are included in the Kiwi Framework, which
 are the base of Delegates and Proxies.
 """
 
+import re
 import string
 
 import gobject
 import gtk
+from gtk import gdk
 
 from kiwi import _warn
 from kiwi.environ import is_gazpacho_required
@@ -41,6 +43,7 @@ from kiwi.interfaces import MixinSupportValidation
 from kiwi.proxies import Proxy
 from kiwi.utils import gsignal, type_register
 from kiwi.ui.gadgets import quit_if_last
+from kiwi.log import Logger
 
 WidgetTree = None
 try:
@@ -58,8 +61,10 @@ except ImportError, e:
             raise RuntimeError("Could not find a glade parser library")
 else:
     from kiwi.ui.gazpacholoader import GazpachoWidgetTree as WidgetTree
-        
-_non_interactive = [
+
+log = Logger('view')
+
+_non_interactive = (
     gtk.Label, 
     gtk.Alignment,
     gtk.AccelLabel,
@@ -81,65 +86,22 @@ _non_interactive = [
     gtk.VPaned,
     gtk.VSeparator,
     gtk.Window, 
-]
+)
 
-_non_interactive = tuple(_non_interactive)
+color_red = gdk.color_parse('red')
+color_black = gdk.color_parse('black')
 
-class signal_block:
-    """
-    A decorator to be used on view methods.
-    It takes a list of arguments which is the name of the widget and
-    the signal name separated by a dot.
-
-    For instance:
-
-        >>> class MyView(SlaveView):
-        ...     @block('money.changed')
-        ...     def update_money(self):
-        ...         self.money.set_value(10)
-        ...     def on_money__changed(self):
-        ...         pass
-
-
-    When calling update_money() the value of the spinbutton called money
-    will be updated, but on_money__changed will not be called.
-    """
-    
-    def __init__(self, *signals):
-        self.signals = []
-        for signal in signals:
-            if not isinstance(signal, str):
-                raise TypeError("signals must be a list of strings")
-            if signal.count('.') != 1:
-                raise TypeError("signal must have exactly one dot")
-            self.signals.append(signal.split('.'))
-
-    def __call__(self, func):
-        def wrapper(view, *args, **kwargs):
-            for name, signal in self.signals:
-                widget = getattr(view, name, None)
-                if widget is None:
-                    raise TypeError("Unknown widget %s in view " % name)
-                view.handler_block(widget, signal)
-                
-            retval = func(view, *args, **kwargs)
-            
-            for name, signal in self.signals:
-                widget = getattr(view, name, None)
-                view.handler_unblock(widget, signal)
-                
-            return retval
-            
-        return wrapper
 #
 # Signal brokers
 #
 
+method_regex = re.compile(r'^(on|after)_(\w+)__(\w+)$')
+
 class SignalBroker(object):
     def __init__(self, view, controller):
         methods = controller._get_all_methods()
         self._do_connections(view, methods)
-
+        
     def _do_connections(self, view, methods):
         """This method allows subclasses to add more connection mechanism"""
         self._autoconnect_by_method_name(view, methods)
@@ -158,34 +120,16 @@ class SignalBroker(object):
         signal name.
         """
         self._autoconnected = {}
+        
         for fname in methods.keys():
             # `on_x__y' has 7 chars and is the smallest possible handler
             # (though illegal, of course, since the signal name x is bogus)
             if len(fname) < 7:
                 continue
-            if fname[:3] == "on_":
-                after = 0
-            elif fname[:6] == "after_":
-                after = 1
-            else:
-                continue
-            # cut out `on_' or `after'
-            f = string.split(fname, "_", 1)
-            if len(f) < 2:
-                continue
-            f = f[1]
-            # separate widget from signal; for instance: 
-            # "on_foo_bar__widget__clicked()" -> "foo_bar", "widget", "clicked"
-            f = string.split(f, "__")
-            if len(f) < 2:
-                continue
-            # signal is the last element
-            signal = f[-1]
-            # widget name is everything up to the last __
-            # "foo_bar", "widget" -> "foo_bar__widget"
-            w_name = string.join(f[:-1], "__")
-            if not w_name:
+            match = method_regex.match(fname)
+            if match is None:
                 continue
+            after, w_name, signal = match.groups()
             widget = getattr(view, w_name, None)
             if widget is None:
                 raise AttributeError("couldn't find widget %s in %s"
@@ -205,10 +149,9 @@ class SignalBroker(object):
                 _warn("Widget %s doesn't provide a signal %s"
                      % (widget.__class__, signal))
                 continue
-            if not self._autoconnected.has_key(widget):
-                self._autoconnected[widget] = []
-            self._autoconnected[widget].append((signal, signal_id))
-
+            self._autoconnected.setdefault(widget, []).append((
+                signal, signal_id))
+            
     def handler_block(self, widget, signal_name):
         signals = self._autoconnected
         if not widget in signals:
@@ -233,9 +176,6 @@ class SignalBroker(object):
                 widget.disconnect(signal_id)
         
 class GladeSignalBroker(SignalBroker):
-    def __init__(self, view, controller):
-        SignalBroker.__init__(self, view, controller)
-
     def _do_connections(self, view, methods):
         super(GladeSignalBroker, self)._do_connections(view, methods)
         self._connect_glade_signals(view, methods)
@@ -254,7 +194,7 @@ class GladeSignalBroker(SignalBroker):
         for name, method in methods.items():
             if callable(method):
                 dict[name] = method
-        view.glade_adaptor.signal_autoconnect(dict)
+        view._glade_adaptor.signal_autoconnect(dict)
         
 
 class SlaveView(gobject.GObject):
@@ -292,6 +232,18 @@ class SlaveView(gobject.GObject):
         """
         gobject.GObject.__init__(self)
 
+        self._broker = None
+        self.slaves = {}
+        self._proxies = []
+        self._valid = True
+
+        # slave/widget name -> validation status
+        self._validation = {}
+        
+        # stores the function that will be called when widgets 
+        # validity is checked
+        self._validate_function = None
+
         # setup the initial state with the value of the arguments or the
         # class variables
         klass = type(self)
@@ -299,94 +251,91 @@ class SlaveView(gobject.GObject):
         self.widgets = widgets or klass.widgets
         self.gladefile = gladefile or klass.gladefile
         self.gladename = gladename or klass.gladename
-        self.toplevel_name = toplevel_name or klass.toplevel_name
+        self.toplevel_name = (toplevel_name or
+                              klass.toplevel_name or
+                              self.gladefile or
+                              self.gladename)
         self.domain = domain or klass.domain
 
-        self.__broker = None
-        
-        for reserved in ["widgets", "toplevel", "gladefile",
-                         "gladename", "tree", "model", "controller"]:
-            # XXX: take into account widget constructor?
-            if reserved in self.widgets:
-                raise AttributeError("The widgets list for %s contains "
-                                     "a widget named `%s', which is "
-                                     "a reserved. name""" % (self, reserved))
-
-
-        self.glade_adaptor = None
-        if self.gladefile is not None:
-            self._init_glade_adaptor()
-
-        # Validation status
-        self._valid = True
-
-        # Dictionary slave/widget name -> validation status
-        self._validation = {}
-        
-        # stores the function that will be called when widgets 
-        # validity is checked
-        self._validate_function = None
-
-        if self.toplevel_name is not None and self.toplevel is None:
-            self.toplevel = getattr(self, self.toplevel_name, None)
-        
-        if not self.toplevel:
-            raise TypeError("A View requires an instance variable "
-                            "called toplevel that specifies the "
-                            "toplevel widget in it")
-
-
-        self.proxies = []
+        self._check_reserved()
+        self._glade_adaptor = self.get_glade_adaptor()
+        self.toplevel = self._get_toplevel()
         
         # grab the accel groups
         self._accel_groups = gtk.accel_groups_from_object(self.toplevel)
 
-        self.slaves = {}
+        # XXX: support normal widgets
+        # notebook page label widget ->
+        #   dict (slave name -> validation status)
+        self._notebook_validation = {}
+        self._notebooks = self._get_notebooks()
 
-    def register_validate_function(self, function):
-        """The signature of the validate function is:
-
-        def function(is_valid):
-
-        or, if it is a method:
-
-        def function(self, is_valid):
-
-        where the 'is_valid' parameter is True if all the widgets have
-        valid data or False otherwise.
-        """
-        self._validate_function = function
-
-    def _init_glade_adaptor(self):
-        """Special init code that subclasses may want to override."""
-        self.glade_adaptor = WidgetTree(self, self.gladefile,
-                                        self.widgets, self.gladename,
-                                        self.domain)
-
-        container_name = self.toplevel_name or self.gladename or self.gladefile
+    def _get_notebooks(self):
+        if not self._glade_adaptor:
+            return []
+        
+        return [widget for widget in self._glade_adaptor.get_widgets()
+                           if isinstance(widget, gtk.Notebook)]
             
-        if container_name is None:
-            msg = ("You provided a gladefile %s to grab the widgets from "
-                   "but you didn't give me a toplevel/container name!")
-            raise ValueError(msg % self.gladefile)
+    def _check_reserved(self):
+        for reserved in ["widgets", "toplevel", "gladefile",
+                         "gladename", "tree", "model", "controller"]:
+            # XXX: take into account widget constructor?
+            if reserved in self.widgets:
+                raise AttributeError(
+                    "The widgets list for %s contains a widget named `%s', "
+                    "which is a reserved. name""" % (self, reserved))
+        
+    def _get_toplevel(self):
+        toplevel = self.toplevel
+        if not toplevel and self.toplevel_name:
+            if self._glade_adaptor:
+                toplevel = self._glade_adaptor.get_widget(self.toplevel_name)
+            else:
+                toplevel = getattr(self, self.toplevel_name, None)
+            
+        if not toplevel:
+            raise TypeError("A View requires an instance variable "
+                            "called toplevel that specifies the "
+                            "toplevel widget in it")
+
+        if isinstance(toplevel, gtk.Window):
+            if toplevel.flags() & gtk.VISIBLE:
+                _warn("Toplevel widget %s (%s) is visible; that's probably "
+                      "wrong" % (toplevel, toplevel.get_name()))
+
+        return toplevel
+    
+    def get_glade_adaptor(self):
+        """Special init code that subclasses may want to override."""
+        if not self.gladefile:
+            return
+            
+        glade_adaptor = WidgetTree(self, self.gladefile,
+                                   self.widgets, self.gladename,
+                                   self.domain)
+
+        container_name = self.toplevel_name
+        if not container_name:
+            raise ValueError(
+                "You provided a gladefile %s to grab the widgets from "
+                "but you didn't give me a toplevel/container name!" %
+                self.gladefile)
 
         # a SlaveView inside a glade file needs to come inside a toplevel
         # window, so we pull our slave out from it, grab its groups and
         # muerder it later
-        shell = self.glade_adaptor.get_widget(container_name)
+        shell = glade_adaptor.get_widget(container_name)
         if not isinstance(shell, gtk.Window):
-            msg = "Container %s should be a Window, found %s"
-            raise TypeError(msg % (container_name, type(shell)))
+            raise TypeError("Container %s should be a Window, found %s" % (
+                container_name, type(shell)))
 
-        if shell.get_property('visible'):
-            _warn('Toplevel window %s in %s should not be visible' % (container_name,
-                                                                      self.gladefile))
-        # XXX grab the accel groups
-        
         self.toplevel = shell.get_child()
         shell.remove(self.toplevel)
         shell.destroy()
 
+        return glade_adaptor
+    
     #
     # Hooks
     #
@@ -445,7 +394,7 @@ class SlaveView(gobject.GObject):
 
     def show_all(self, *args):
         """Shows all widgets attached to the toplevel widget"""
-        if self.glade_adaptor is not None:
+        if self._glade_adaptor is not None:
             raise AssertionError("You don't want to call show_all on a "
                                  "SlaveView. Use show() instead.")
         self.toplevel.show_all()
@@ -482,7 +431,8 @@ class SlaveView(gobject.GObject):
               self.widgets is None, looks through all widgets attached
               to the view.
 
-            - can_focus: boolean, if set only searches through widget that can focus
+            - can_focus: boolean, if set only searches through widget
+              that can be focused
         """
         # XXX: recurse through containers from toplevel widget, better
         # idea and will work.
@@ -528,12 +478,12 @@ class SlaveView(gobject.GObject):
     #
 
     def _attach_callbacks(self, controller):
-        if self.glade_adaptor is None:
+        if self._glade_adaptor is None:
             brokerclass = SignalBroker
         else:
             brokerclass = GladeSignalBroker
             
-        self.__broker = brokerclass(self, controller)
+        self._broker = brokerclass(self, controller)
 
 #    def _setup_keypress_handler(self, keypress_handler):
 #        # Only useful in BaseView and derived classes
@@ -565,6 +515,10 @@ class SlaveView(gobject.GObject):
         instead of the eventbox) is still supported for compatibility
         reasons but will print a warning.
         """
+        log('%s: Attaching slave %s of type %s' % (self.__class__.__name__,
+                                                   name,
+                                                   slave.__class__.__name__))
+            
         if name in self.slaves:
             # XXX: TypeError
             _warn("A slave with name %s is already attached to %r"  % (
@@ -572,7 +526,8 @@ class SlaveView(gobject.GObject):
         self.slaves[name] = slave
 
         if not isinstance(slave, SlaveView):
-            raise TypeError("slave must be a SlaveView, not a %s" % type(slave))
+            raise TypeError("slave must be a SlaveView, not a %s" %
+                            type(slave))
 
         shell = slave.get_toplevel()
 
@@ -584,8 +539,8 @@ class SlaveView(gobject.GObject):
 
         # if our widgets are in a glade file get the placeholder from them
         # or take it from the view itself otherwise
-        if self.glade_adaptor:
-            placeholder = self.glade_adaptor.get_widget(name)
+        if self._glade_adaptor:
+            placeholder = self._glade_adaptor.get_widget(name)
         else:
             placeholder = getattr(self, name, None)
             
@@ -597,7 +552,8 @@ class SlaveView(gobject.GObject):
         if slave._accel_groups:
             # take care of accelerator groups; attach to parent window if we
             # have one; if embedding a slave into another slave, store its
-            # accel groups; otherwise complain if we're dropping the accelerators
+            # accel groups; otherwise complain if we're dropping the
+            # accelerators
             win = parent.get_toplevel()
             if isinstance(win, gtk.Window):
                 # use idle_add to be sure we attach the groups as late
@@ -634,6 +590,18 @@ class SlaveView(gobject.GObject):
         slave.connect_object('validation-changed',
                              self._on_child__validation_changed,
                              name)
+
+        for notebook in self._notebooks:
+            for child in notebook.get_children():
+                if not shell.is_ancestor(child):
+                    continue
+                
+                label = notebook.get_tab_label(child)
+                slave.connect('validation-changed',
+                              self._on_notebook_slave__validation_changed,
+                              name, label)
+                self._notebook_validation[label] = {}
+
         # Fire of an initial notification
         slave.check_and_notify_validity(force=True)
         
@@ -684,7 +652,8 @@ class SlaveView(gobject.GObject):
             - widgets: a list of GtkWidgets
             - signal: a string specifying the signals
             - handler: a callback method
-            - after: a boolean; if TRUE, we use connect_after(), otherwise, connect()
+            - after: a boolean; if TRUE, we use connect_after(), otherwise,
+            connect()
         """
         if not isinstance(widgets, (list, tuple)):
             raise TypeError("widgets must be a list, found %s" % widgets)
@@ -701,18 +670,18 @@ class SlaveView(gobject.GObject):
         """
         Disconnect handlers previously connected with 
         autoconnect_signals()"""
-        self.__broker.disconnect_autoconnected()
+        self._broker.disconnect_autoconnected()
         
     def handler_block(self, widget, signal_name=None):
         # XXX: Warning, or bail out?
-        if not self.__broker:
+        if not self._broker:
             return
-        self.__broker.handler_block(widget, signal_name)
+        self._broker.handler_block(widget, signal_name)
 
     def handler_unblock(self, widget, signal_name=None):
-        if not self.__broker:
+        if not self._broker:
             return
-        self.__broker.handler_unblock(widget, signal_name)
+        self._broker.handler_unblock(widget, signal_name)
         
     #
     # Proxies
@@ -724,18 +693,22 @@ class SlaveView(gobject.GObject):
         the view changes. Arguments:
           
           - model. the object we are proxing. It can be None if we don't have
-          a model yet and we want to display the interface and set it up with
-          future models.
+            a model yet and we want to display the interface and set it up with
+            future models.
           - widgets. the list of widgets that contains model attributes to be
-          proxied. If it is None (or not specified) it will be the whole list
-          of widgets this View has.
+            proxied. If it is None (or not specified) it will be the whole list
+            of widgets this View has.
 
         This method return a Proxy object that you may want to use to force
         updates or setting new models. Keep a reference to it since there is
         no way to get that proxy later on. You have been warned (tm)
         """
+        log('%s: adding proxy for %s' % (
+            self.__class__.__name__, 
+            model and model.__class__.__name__))
+        
         widgets = widgets or self.widgets
-
+        
         for widget_name in widgets:
             widget = getattr(self, widget_name, None)
             if (widget is None or
@@ -749,10 +722,9 @@ class SlaveView(gobject.GObject):
             except TypeError:
                 raise AssertionError("%r does not have a validation-changed "
                                      "signal." % widget)
-
+            
         proxy = Proxy(self, model, widgets)
-        self.proxies.append(proxy)
-
+        self._proxies.append(proxy)
         return proxy
 
     #
@@ -760,12 +732,32 @@ class SlaveView(gobject.GObject):
     #
     
     def _on_child__validation_changed(self, name, value):
-        # Children of the view, eg slaves or widgets are connected to this signal
-        # When validation changes of a validatable child this callback is called
+        # Children of the view, eg slaves or widgets are connected to
+        # this signal. When validation changes of a validatable child
+        # this callback is called
         self._validation[name] = value
 
         self.check_and_notify_validity()
         
+    def _on_notebook_slave__validation_changed(self, slave, value, name,
+                                               label):
+        validation = self._notebook_validation[label]
+        validation[name] = value
+
+        is_valid = True
+        if False in validation.values():
+            is_valid = False
+            
+        if is_valid:
+            color = color_black
+        else:
+            color = color_red
+
+        # Only modify active state, since that's the (somewhat badly named)
+        # state used for the pages which are not selected.
+        label.modify_fg(gtk.STATE_ACTIVE, color)
+        label.modify_fg(gtk.STATE_NORMAL, color)
+        
     def check_and_notify_validity(self, force=False):
         # Current view is only valid if we have no invalid children
         # their status are stored as values in the dictionary
@@ -787,47 +779,49 @@ class SlaveView(gobject.GObject):
     def force_validation(self):
         self.check_and_notify_validity(force=True)
         
+    def register_validate_function(self, function):
+        """The signature of the validate function is:
+
+        def function(is_valid):
+
+        or, if it is a method:
+
+        def function(self, is_valid):
+
+        where the 'is_valid' parameter is True if all the widgets have
+        valid data or False otherwise.
+        """
+        self._validate_function = function
+
 type_register(SlaveView)
 
 class BaseView(SlaveView):
     """A view with a toplevel window."""
     
-    def __init__(self, toplevel=None, delete_handler=None, widgets=None,
-                 gladefile=None, gladename=None, toplevel_name=None,
-                 domain=None):
-        """ toplevel is the widget to be set as `toplevel' (and which will
-        be aliased as `win'); delete_handler allows setting a function
-        to be called when this view's window is deleted."""
-        try:
-            SlaveView.__init__(self, toplevel, widgets, gladefile, gladename,
-                               toplevel_name, domain)
-        except KeyError:
-            raise KeyError("Some widgets were defined in self.widgets "
-                           "but not found in the glade tree (see previous "
-                           "messages to see which ones).")
-            
+    def __init__(self, toplevel=None, widgets=None, gladefile=None,
+                 gladename=None, toplevel_name=None, domain=None,
+                 delete_handler=None):
+        SlaveView.__init__(self, toplevel, widgets, gladefile, gladename,
+                           toplevel_name, domain)
 
         if not isinstance(self.toplevel, gtk.Window):
             raise TypeError("toplevel widget must be a Window "
                             "(or inherit from it),\nfound `%s' %s" 
                             % (toplevel, self.toplevel))
-
+        self.toplevel.set_name(self.__class__.__name__)
+        
         if delete_handler:
-            id = self.toplevel.connect("delete_event", delete_handler)
+            id = self.toplevel.connect("delete-event", delete_handler)
             if not id:
                 raise ValueError(
                     "Invalid delete handler provided: %s" % delete_handler)
 
-    def _init_glade_adaptor(self):
-        self.glade_adaptor = WidgetTree(self, self.gladefile,
-                                        self.widgets, self.gladename)
-        name = self.toplevel_name or self.glade_adaptor.gladename
-        self.toplevel = self.glade_adaptor.get_widget(name)
-
-        if self.toplevel.flags() & gtk.VISIBLE:
-            _warn("Toplevel widget %s (%s) is visible; that's probably "
-                  "wrong" % (self.toplevel, name))
-            
+    def get_glade_adaptor(self):
+        if not self.gladefile:
+            return
+        
+        return WidgetTree(self, self.gladefile, self.widgets,
+                          self.gladename)
     
     #
     # Hook for keypress handling
