@@ -31,14 +31,11 @@ support and display an icon using L{kiwi.ui.icon.IconEntry}.
 
 import gettext
 
-import gobject
 import gtk
 
-from kiwi import ValueUnset
-from kiwi.interfaces import implementsIProxy, implementsIMandatoryProxy
 from kiwi.ui.icon import IconEntry
 from kiwi.ui.widgets.proxy import WidgetMixinSupportValidation
-from kiwi.utils import gproperty, gsignal, type_register
+from kiwi.utils import PropertyObject, gproperty, gsignal, type_register
 
 _ = gettext.gettext
 
@@ -48,7 +45,7 @@ _ = gettext.gettext
 (ENTRY_MODE_TEXT,
  ENTRY_MODE_DATA) = range(2)
 
-class Entry(gtk.Entry, WidgetMixinSupportValidation):
+class Entry(PropertyObject, gtk.Entry, WidgetMixinSupportValidation):
     """The Kiwi Entry widget has many special features that extend the basic
     gtk entry.
     
@@ -63,22 +60,17 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
     entry. When dealing with date and float data-type the information on
     how to fill these entries is displayed according to the current locale.
     """
-    implementsIProxy()
-    implementsIMandatoryProxy()
 
-    gproperty("completion", bool, False, 
-              "Completion", gobject.PARAM_READWRITE)
-    gproperty('exact-completion', bool, True)
+    gproperty("completion", bool, False)
+    gproperty('exact-completion', bool, default=False)
         
     def __init__(self):
         gtk.Entry.__init__(self)
         WidgetMixinSupportValidation.__init__(self)
-        self._completion = False
+        PropertyObject.__init__(self)
         self._current_object = None
         self._entry_mode = ENTRY_MODE_TEXT
-        self._exact_completion = True
         self._icon = IconEntry(self)
-        self.show()
 
     # Virtual methods
     gsignal('changed', 'override')
@@ -95,28 +87,20 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
         self.emit('content-changed')
 
     # Properties
-    
-    def prop_get_exact_completion(self):
-        return self._exact_completion
-
     def prop_set_exact_completion(self, value):
-        self._exact_completion = value
-        
         if value:
             match_func = self._completion_exact_match_func
         else:
             match_func = self._completion_normal_match_func
         completion = self._create_completion()
         completion.set_match_func(match_func)
-                
-    def prop_get_completion(self):
-        return self._completion
+
+        return value
     
     def prop_set_completion(self, value):
-        self._completion = value
-
         if not self.get_completion():
             self._enable_completion()
+        return value
     
     # Public API
     def set_exact_completion(self, value):
@@ -129,8 +113,7 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
         @type value:  boolean
         """
         
-        self.prop_set_exact_completion(value)
-        self.notify('exact-completion')
+        self.exact_completion = value
 
     # XXX: Decide if this API or the Combobox prefill API should be used
     def set_completion_strings(self, strings=[], values=[]):
@@ -175,8 +158,7 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
     def _update_current_object(self, text):
         if self._entry_mode != ENTRY_MODE_DATA:
             return
-        
-        obj = None
+
         for row in self.get_completion().get_model():
             if row[COL_TEXT] == text:
                 self._current_object = row[COL_OBJECT]
@@ -185,8 +167,7 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
             # Customized validation
             if text:
                 self.set_invalid(_("'%s' is not a valid object" % text))
-            elif self._mandatory:
-                self._fade.stop()
+            elif self.mandatory:
                 self.set_blank()
             else:
                 self.set_valid()
@@ -215,7 +196,7 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
         self.set_completion(completion)
         completion.set_model(gtk.ListStore(str, object))
         completion.set_text_column(0)
-        self.prop_set_exact_completion(False)
+        self.exact_completion = False
         completion.connect("match-selected",
                            self._on_completion__match_selected)
         self._current_object = None
@@ -241,36 +222,35 @@ class Entry(gtk.Entry, WidgetMixinSupportValidation):
         if not len(model):
             return
 
-        text, data = model[iter]
-        self.set_text(text)
+        # this updates current_object and triggers content-changed
+        self.set_text(model[iter][COL_TEXT]) 
         self.set_position(-1)
-        self._current_object = data
-        self.emit('content-changed')
+        # FIXME: Enable this at some point
+        #self.activate()
         
     def read(self):
         mode = self._entry_mode
         if mode == ENTRY_MODE_TEXT:
-            return self.get_text()
+            return self._from_string(self.get_text())
         elif mode == ENTRY_MODE_DATA:
             return self._current_object
         else:
             raise AssertionError
     
     def update(self, data):
-        if data is ValueUnset or data is None:
-            self.set_text("")
-            return
-        
-        mode = self._entry_mode
-        if mode == ENTRY_MODE_DATA:
-            new = self._get_text_from_object(data)
-            if new is None:
-                raise TypeError("%r is not a data object" % data)
-            data = new
-        elif mode == ENTRY_MODE_TEXT:
-            data = self.type2str(data)
+        if data is None:
+            text = ""
+        else:
+            mode = self._entry_mode
+            if mode == ENTRY_MODE_DATA:
+                new = self._get_text_from_object(data)
+                if new is None:
+                    raise TypeError("%r is not a data object" % data)
+                text = new
+            elif mode == ENTRY_MODE_TEXT:
+                text = self._as_string(data)
 
-        self.set_text(data)
+        self.set_text(text)
 
     def do_expose_event(self, event):
         gtk.Entry.do_expose_event(self, event)
