@@ -33,15 +33,16 @@ import gobject
 import gtk
 from gtk import gdk
 
-from kiwi import _warn, datatypes, ValueUnset
+from kiwi import _warn, ValueUnset
 from kiwi.accessors import kgetattr
+from kiwi.datatypes import converter, currency, lformat
 from kiwi.decorators import deprecated
 from kiwi.python import slicerange
 from kiwi.utils import PropertyObject, gsignal, gproperty, type_register
 
 _ = gettext.gettext
 
-str2type = datatypes.converter.str_to_type
+str2type = converter.str_to_type
 
 def str2enum(value_name, enum_class):
     "converts a string to a enum"
@@ -49,10 +50,9 @@ def str2enum(value_name, enum_class):
         if value_name in (enum.value_name, enum.value_nick):
             return enum
 
-def str2bool(value, default_value=False,
-             from_string=datatypes.converter.from_string):
+def str2bool(value, from_string=converter.from_string):
     "converts a boolean to a enum"
-    return from_string(bool, value, default_value)
+    return from_string(bool, value)
 
 class Column(PropertyObject, gobject.GObject):
     """Specifies a column in a List"""
@@ -146,11 +146,20 @@ class Column(PropertyObject, gobject.GObject):
             raise AttributeError(msg)
  
         self.attribute = attribute
+        self.compare = None
+        self.as_string = None
+        self.from_string = None
 
         kwargs['title'] = title or attribute.capitalize()
         if data_type:
             kwargs['data_type'] = data_type
 
+        # If we don't specify a justification, right align it for int/float
+        # and left align it for everything else. 
+        if "justify" not in kwargs:
+            if data_type and issubclass(data_type, (int, float, long, currency)):
+                kwargs['justify'] = gtk.JUSTIFY_RIGHT
+                
         format_func = kwargs.get('format_func')
         if format_func:
             if not callable(format_func):
@@ -165,12 +174,20 @@ class Column(PropertyObject, gobject.GObject):
     # This is meant to be subclassable, we're using kgetattr, as
     # a staticmethod as an optimization, so we can avoid a function call.
     get_attribute = staticmethod(kgetattr)
-                                             
+
+    def prop_set_data_type(self, data):
+        if data is not None:
+            conv = converter.get_converter(data)
+            self.compare = conv.get_compare_function()
+            self.as_string = conv.as_string
+            self.from_string = conv.from_string
+        return data
+    
     def __repr__(self):
-        ns = self.__dict__.copy()
-        attr = ns['attribute']
-        del ns['attribute']
-        return "<%s %s: %s>" % (self.__class__.__name__, attr, ns)
+        namespace = self.__dict__.copy()
+        attr = namespace['attribute']
+        del namespace['attribute']
+        return "<%s %s: %s>" % (self.__class__.__name__, attr, namespace)
 
     # XXX: Replace these two with a gazpacho loader adapter
     def __str__(self):
@@ -197,7 +214,7 @@ class Column(PropertyObject, gobject.GObject):
         column = cls(fields[0])
         column.title = fields[1] or ''
         column.data_type = str2type(fields[2])
-        column.visible = str2bool(fields[3], default_value=True)
+        column.visible = str2bool(fields[3])
         column.justify = str2enum(fields[4], gtk.JUSTIFY_LEFT)
         column.tooltip = fields[5]
         column.format = fields[6]
@@ -207,9 +224,10 @@ class Column(PropertyObject, gobject.GObject):
         except ValueError:
             pass
         
-        column.sorted = str2bool(fields[8], default_value=False)
+        column.sorted = str2bool(fields[8])
         column.order = str2enum(fields[9], gtk.SORT_ASCENDING) \
                      or gtk.SORT_ASCENDING
+        
         # XXX: expand, remember to sync with __str__
         
         return column
@@ -226,13 +244,13 @@ class SequentialColumn(Column):
     right justify the sequences."""
     def __init__(self, title='#', justify=gtk.JUSTIFY_RIGHT, **kwargs):
         Column.__init__(self, '_kiwi_sequence_id',
-                        title=title, data_type=int, **kwargs)
+                        title=title, justify=justify, data_type=int, **kwargs)
 
-    def cell_data_func(self, tree_column, renderer, model, iter,
-                       (column, renderer_prop)):
+    def cell_data_func(self, tree_column, renderer, model, treeiter,
+                       (column, renderer_prop, as_string)):
         reversed = tree_column.get_sort_order() == gtk.SORT_DESCENDING
 
-        row = model[iter]
+        row = model[treeiter]
         if reversed:
             sequence_id = len(model) - row.path[0]
         else:
@@ -364,14 +382,19 @@ COL_MODEL = 0
 
 _marker = object()
 
-class List(gtk.ScrolledWindow):
+class List(PropertyObject, gtk.ScrolledWindow):
     """An enhanced version of GtkTreeView, which provides pythonic wrappers
     for accessing rows, and optional facilities for column sorting (with
     types) and column selection."""
-    
+
+    # selected row(s)
     gsignal('selection-changed', object)
+    
+    # row clicked
     gsignal('double-click', object)
-    gsignal('cell-edited', str, object)
+    
+    # edited object, attribute name
+    gsignal('cell-edited', object, str)
 
     # this property is used to serialize the columns of a List. The format
     # is a big string with '^' as the column separator and '|' as the field
@@ -400,23 +423,24 @@ class List(gtk.ScrolledWindow):
         # so we can't do this check.
         #elif mode == gtk.SELECTION_EXTENDED:
         #    raise TypeError("gtk.SELECTION_EXTENDED is deprecated")
-        
-        gtk.ScrolledWindow.__init__(self)
-        # we always want a vertical scrollbar. Otherwise the button on top
-        # of it doesn't make sense. This button is used to display the popup
-        # menu
-        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
 
         # Mapping of instance id -> treeiter
         self._iters = {}
         self._cell_data_caches = {}
         self._columns_configured = False
         self._autosize = True
-
+        self._vscrollbar = None
         # by default we are unordered. This index points to the column
         # definition of the column that dictates the order, in case there is
         # any
         self._sort_column_index = -1
+
+        gtk.ScrolledWindow.__init__(self)
+                                
+        # we always want a vertical scrollbar. Otherwise the button on top
+        # of it doesn't make sense. This button is used to display the popup
+        # menu
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
         
         self._model = gtk.ListStore(object)
         self._model.set_sort_func(COL_MODEL, self._sort_function)
@@ -456,9 +480,10 @@ class List(gtk.ScrolledWindow):
         # Select the first item if no items are selected
         if mode != gtk.SELECTION_NONE and instance_list:
             selection.select_iter(self._model[COL_MODEL].iter)
-            
-        self.show()
 
+        # Depends on treeview and selection being set up
+        PropertyObject.__init__(self)
+                    
     # Python list object implementation
     # These methods makes the kiwi list behave more or less
     # like a normal python list
@@ -529,18 +554,28 @@ class List(gtk.ScrolledWindow):
     # append and remove are below
 
     def extend(self, iterable):
-        "L.extend(iterable) -- extend list by appending elements from the iterable"
+        """
+        Extend list by appending elements from the iterable
+        
+        @param iterable:
+        """
         
         return self.add_list(iterable, clear=False)
 
     def index(self, item, start=None, stop=None):
-        "L.index(item, [start, [stop]]) -> integer -- return first index of value"
+        """
+        Return first index of value
+        
+        @param item:
+        @param start:
+        @param stop
+        """
         
         if start or stop:
             raise NotImplementedError("start and stop")
 
-        iter = self._iters.get(id(item), _marker)
-        if iter is _marker:
+        treeiter = self._iters.get(id(item), _marker)
+        if treeiter is _marker:
             raise ValueError("item %r is not in the list" % item)
         
         return self._model[item.iter].path[0]
@@ -559,7 +594,10 @@ class List(gtk.ScrolledWindow):
         raise NotImplementedError
     
     def pop(self, index):
-        "L.pop([index]) -> item -- remove and return item at index (default last)"
+        """
+        Remove and return item at index (default last)
+        @param index:
+        """
         raise NotImplementedError
 
     def reverse(self, pos, item):
@@ -571,23 +609,18 @@ class List(gtk.ScrolledWindow):
         cmp(x, y) -> -1, 0, 1"""
         raise NotImplementedError
 
-    # GObject property handling
-    def do_get_property(self, pspec):
-        if pspec.name == 'column-definitions':
-            return self._columns_string
-        elif pspec.name == 'selection-mode':
-            return self.get_selection_mode()
-        else:
-            raise AttributeError('Unknown property %s' % pspec.name)
-
-    def do_set_property(self, pspec, value):
-        if pspec.name == 'column-definitions':
-            self.set_columns(value)
-        elif pspec.name == 'selection-mode':
-            self.set_selection_mode(value)
-        else:
-            raise AttributeError('Unknown property %s' % pspec.name)
+    # Properties
+    
+    def prop_set_column_definition(self, value):
+        self.set_columns(value)
+        return value
+    
+    def prop_set_selection_mode(self, mode):
+        self.set_selection_mode(mode)
 
+    def prop_get_selection_mode(self):
+        return self.get_selection_mode()
+    
     # Columns handling
     def _load(self, instances, progress_handler=None):
         # do nothing if empty list or None provided
@@ -639,12 +672,12 @@ class List(gtk.ScrolledWindow):
             raise TypeError("Failed to get attribute '%s' for %s" %
                             (column.attribute, instance))
             
-        tp = type(value)
-        if tp is type(None):
+        value_type = type(value)
+        if value_type is type(None):
             raise TypeError("Detected invalid type None for column `%s'; "
                             "please specify type in Column constructor.""" %
                             column.attribute)
-        return tp
+        return value_type
     
     def _setup_columns(self):
         if self._columns_configured:
@@ -685,14 +718,6 @@ class List(gtk.ScrolledWindow):
         if column.on_attach_renderer:
             column.on_attach_renderer(renderer)
         justify = column.justify
-        # If we don't specify a justification, right align it for int/float
-        # and left align it for everything else. 
-        if justify is None:
-            if issubclass(column.data_type, (int, float)):
-                justify = gtk.JUSTIFY_RIGHT
-            else:
-                justify = gtk.JUSTIFY_LEFT
-                
         if justify == gtk.JUSTIFY_RIGHT:
             xalign = 1.0
         elif justify == gtk.JUSTIFY_CENTER:
@@ -713,7 +738,8 @@ class List(gtk.ScrolledWindow):
             
         treeview_column.pack_start(renderer)
         treeview_column.set_cell_data_func(renderer, cell_data_func,
-                                           (column, renderer_prop))
+                                           (column, renderer_prop,
+                                            column.as_string))
         treeview_column.set_visible(column.visible)
 
         treeview_column.connect("clicked", self._on_column__clicked, column)
@@ -783,7 +809,13 @@ class List(gtk.ScrolledWindow):
                        self._on_header__button_release_event)
         return treeview_column
     
-    def _on_renderer_toggle__toggled(self, renderer, path, model, attribute):
+    def _on_renderer_toggle_check__toggled(self, renderer, path, model, attribute):
+        obj = model[path][COL_MODEL]
+        value = not getattr(obj, attribute, None)
+        setattr(obj, attribute, value)
+        self.emit('cell-edited', obj, attribute)
+
+    def _on_renderer_toggle_radio__toggled(self, renderer, path, model, attribute):
         # Deactive old one
         old = renderer.get_data('kiwilist::radio-active')
         
@@ -810,13 +842,14 @@ class List(gtk.ScrolledWindow):
         new = model[path][COL_MODEL]
         setattr(new, attribute, True)
         renderer.set_data('kiwilist::radio-active', new)
+        self.emit('cell-edited', new, attribute)
         
     def _on_renderer_text__edited(self, renderer, path, text,
-                                  model, attribute, column):
+                                  model, attribute, column, as_string):
         obj = model[path][COL_MODEL]
-        value = datatypes.converter.from_string(column.data_type, text)
+        value = as_string(text)
         setattr(obj, attribute, value)
-        self.emit('cell-edited', attribute, value)
+        self.emit('cell-edited', obj, attribute)
         
     def _guess_renderer_for_type(self, column):
         """Gusses which CellRenderer we should use for a given type.
@@ -828,40 +861,43 @@ class List(gtk.ScrolledWindow):
         data_type = column.data_type
         if data_type is bool:
             renderer = gtk.CellRendererToggle()
-            if column.radio:
-                renderer.set_radio(True)
-
             if column.editable:
                 renderer.set_property('activatable', True)
-                renderer.connect('toggled', self._on_renderer_toggle__toggled,
-                                 self._model, column.attribute)
+
+            if column.radio:
+                renderer.set_radio(True)
+                cb = self._on_renderer_toggle_radio__toggled
+            else:
+                cb = self._on_renderer_toggle_check__toggled
+            renderer.connect('toggled', cb, self._model, column.attribute)
                 
             prop = 'active'
         elif issubclass(data_type, (datetime.date, datetime.time,
-                                  basestring, int, float)):
+                                    basestring, int, float)):
             renderer = gtk.CellRendererText()
             prop = 'text'
             if column.editable:
                 renderer.set_property('editable', True)
                 renderer.connect('edited', self._on_renderer_text__edited,
-                                 self._model, column.attribute, column)
+                                 self._model, column.attribute, column,
+                                 column.from_string)
 
         else:
             raise ValueError("the type %s is not supported yet" % data_type)
 
         return renderer, prop
     
-    def _search_equal_func(self, model, tree_column, key, iter, column):
-        data = column.get_attribute(model[iter][COL_MODEL],
+    def _search_equal_func(self, model, tree_column, key, treeiter, column):
+        data = column.get_attribute(model[treeiter][COL_MODEL],
                                     column.attribute, None)
         if data.startswith(key):
             return False
         return True
         
-    def _cell_data_func(self, tree_column, renderer, model, iter,
-                        (column, renderer_prop)):
+    def _cell_data_func(self, tree_column, renderer, model, treeiter,
+                        (column, renderer_prop, as_string)):
 
-        row = model[iter]
+        row = model[treeiter]
         if column.cache:
             cache = self._cell_data_caches[column.attribute]
             path = row.path[0]
@@ -874,20 +910,16 @@ class List(gtk.ScrolledWindow):
         else:
             data = column.get_attribute(row[COL_MODEL],
                                         column.attribute, None)
-            
+
         if column.format:
-            text = datatypes.lformat(column.format, data)
+            text = lformat(column.format, data)
         elif column.format_func:
             text = column.format_func(data)
-        elif column.data_type == datetime.date:
-            # date according to current locale
-            text = data.strftime('%x') 
-        elif column.data_type == datetime.time:
-            # time according to current locale
-            text = data.strftime('%X')
-        elif column.data_type == datetime.datetime:
-            # date and time according to current locale
-            text = data.strftime('%c')
+        elif (column.data_type == datetime.date or
+              column.data_type == datetime.datetime or
+              column.data_type == datetime.time or
+              column.data_type == currency):
+            text = as_string(data)
         else:
             text = data
             
@@ -930,8 +962,9 @@ class List(gtk.ScrolledWindow):
     def _sort_function(self, model, iter1, iter2):
         column = self._columns[self._sort_column_index]
         attr = column.attribute
-        return cmp(column.get_attribute(model[iter1][COL_MODEL], attr),
-                   column.get_attribute(model[iter2][COL_MODEL], attr))
+        return column.compare(
+            column.get_attribute(model[iter1][COL_MODEL], attr),
+            column.get_attribute(model[iter2][COL_MODEL], attr))
 
     def _on_column__clicked(self, treeview_column, column):
         if self._sort_column_index == -1:
@@ -1046,8 +1079,9 @@ class List(gtk.ScrolledWindow):
         # put the popup_window in its position
         gdk_window = self.window
         if gdk_window:
-            x, y = gdk_window.get_origin()
-            self._popup_window.move(x + old_alloc.x, y + old_alloc.y)
+            winx, winy = gdk_window.get_origin()
+            self._popup_window.move(winx + old_alloc.x,
+                                    winy + old_alloc.y)
         
     # end of the popup button hack
 
@@ -1101,8 +1135,7 @@ class List(gtk.ScrolledWindow):
             for col in value.split('^'):
                 if not col:
                     continue
-                c = Column.from_string(col)
-                self._columns.append(c)
+                self._columns.append(Column.from_string(col))
         elif isinstance(value, (list, tuple)):
             self._columns = value
             self._columns_string = '^'.join([str(col) for col in value])
@@ -1152,13 +1185,14 @@ class List(gtk.ScrolledWindow):
         # linear search for the instance to remove
         treeiter = self._iters.pop(objid)
         if treeiter:
-            self._model.remove(treeiter)
-            path = self._model[treeiter].path[0]
-            
             # Remove any references to this path
+            path = self._model[treeiter].path[0]
             for cache in self._cell_data_caches.values():
                 if path in cache:
                     del cache[path]
+
+            # All references to the iter gone, now it can be removed
+            self._model.remove(treeiter)
                 
             return True
             
@@ -1201,23 +1235,37 @@ class List(gtk.ScrolledWindow):
         selection = self._treeview.get_selection()
         if selection:
             selection.unselect_all()
-            
-    def select(self, instance, scroll=True):
-        objid = id(instance)
-        if not objid in self._iters:
-            raise ValueError("instance %r is not in the list" % instance)
-        iter = self._iters[objid]
+
+    def select_paths(self, paths):
+        """
+        Selects a number of rows corresponding to paths
+        @param paths:
+        """
 
         selection = self._treeview.get_selection()
         if selection.get_mode() == gtk.SELECTION_NONE:
             raise TypeError("Selection not allowed")
+
+        for path in paths:
+            selection.select_path(path)
+
+    def select(self, instance, scroll=True):
+        objid = id(instance)
+        if not objid in self._iters:
+            raise ValueError("instance %r is not in the list" % instance)
+
+        selection = self._treeview.get_selection()
+        if selection.get_mode() == gtk.SELECTION_NONE:
+            raise TypeError("Selection not allowed")
+
+        treeiter = self._iters[objid]
         
-        selection.select_iter(iter)
+        selection.select_iter(treeiter)
 
         if scroll:
-            self._treeview.scroll_to_cell(self._model[iter].path,
+            self._treeview.scroll_to_cell(self._model[treeiter].path,
                                           None, True, 0.5, 0)
-                                      
+
     def get_selected(self):
         """Returns the currently selected object
         If an object is not selected, None is returned
@@ -1233,9 +1281,9 @@ class List(gtk.ScrolledWindow):
         elif mode not in (gtk.SELECTION_SINGLE, gtk.SELECTION_BROWSE):
             _warn('get_selected() called when multiple rows can be selected')
 
-        model, iter = selection.get_selected()
-        if iter:
-            return model[iter][COL_MODEL]
+        model, treeiter = selection.get_selected()
+        if treeiter:
+            return model[treeiter][COL_MODEL]
 
     def get_selected_rows(self):
         """Returns a list of currently selected objects
@@ -1258,15 +1306,16 @@ class List(gtk.ScrolledWindow):
             return [model[path][COL_MODEL] for (path,) in paths]
         return []
     
-    def add_list(self, list, clear=True, progress_handler=None):
+    def add_list(self, instances, clear=True, progress_handler=None):
         """
         Allows a list to be loaded, by default clearing it first.
         freeze() and thaw() are called internally to avoid flashing.
         
-          - list: a list to be added
-          - clear: a boolean that specifies whether or not to clear the list
-          - progress_handler: a callback function to be called while the list
-            is being filled
+        @param instances: a list to be added
+        @param clear: a boolean that specifies whether or not to
+          clear the list
+        @param progress_handler: a callback function to be called
+            while the list is being filled
         """
 
         self._treeview.freeze_notify()
@@ -1275,7 +1324,7 @@ class List(gtk.ScrolledWindow):
             self.unselect_all()
             self.clear()
             
-        ret = self._load(list, progress_handler)
+        ret = self._load(instances, progress_handler)
             
         self._treeview.thaw_notify()
         return ret
@@ -1302,10 +1351,11 @@ class List(gtk.ScrolledWindow):
         objid = id(instance)
         if not objid in self._iters:
             raise ValueError("instance %r is not in the list" % instance)
-        iter = self._iters[objid]
+
+        treeiter = self._iters[objid]
         
         model = self._model
-        pos = model[iter].path[0]
+        pos = model[treeiter].path[0]
         if pos >= len(model) - 1:
             pos = 0
         else:
@@ -1324,10 +1374,10 @@ class List(gtk.ScrolledWindow):
         objid = id(instance)
         if not objid in self._iters:
             raise ValueError("instance %r is not in the list" % instance)
-        iter = self._iters[objid]
+        treeiter = self._iters[objid]
         
         model = self._model
-        pos = model[iter].path[0]
+        pos = model[treeiter].path[0]
         if pos == 0:
             pos = len(model) - 1
         else:
@@ -1480,11 +1530,13 @@ class SummaryLabel(ListLabel):
     def update_total(self):
         """Recalculate the total value of all columns"""
         attr = self._column.attribute
-        value = sum([kgetattr(obj, attr) for obj in self._klist], 0.0)
+        get_attribute = self._column.get_attribute
+
+        value = sum([get_attribute(obj, attr) for obj in self._klist], 0.0)
         self.set_value(value)
 
     # Callbacks
     
-    def _on_klist__cell_edited(self, klist, attribute, value):
+    def _on_klist__cell_edited(self, klist, object, attribute):
         self.update_total()
 
